@@ -5,6 +5,7 @@ import pathspec
 
 from hatchling.builders.constants import DEFAULT_BUILD_DIRECTORY, EXCLUDED_DIRECTORIES, BuildEnvVars
 from hatchling.builders.utils import normalize_inclusion_map, normalize_relative_directory, normalize_relative_path
+from hatchling.metadata.utils import normalize_project_name
 from hatchling.utils.fs import locate_file
 
 
@@ -24,7 +25,7 @@ class BuilderConfig:
         self.__force_include = None
         self.__vcs_exclusion_files = None
 
-        # Possible pathspec.PathSpec
+        # Possible pathspec.GitIgnoreSpec
         self.__include_spec = None
         self.__exclude_spec = None
         self.__artifact_spec = None
@@ -48,6 +49,7 @@ class BuilderConfig:
         self.__dev_mode_dirs = None
         self.__dev_mode_exact = None
         self.__require_runtime_dependencies = None
+        self.__require_runtime_features = None
 
     @property
     def builder(self):
@@ -149,9 +151,7 @@ class BuilderConfig:
                 all_include_patterns.append(f"/{relative_path.replace(os.sep, '/')}/")
 
             if all_include_patterns:
-                self.__include_spec = pathspec.PathSpec.from_lines(
-                    pathspec.patterns.GitWildMatchPattern, all_include_patterns
-                )
+                self.__include_spec = pathspec.GitIgnoreSpec.from_lines(all_include_patterns)
 
             self.__include_patterns = all_include_patterns
 
@@ -185,9 +185,7 @@ class BuilderConfig:
                 all_exclude_patterns.extend(self.load_vcs_exclusion_patterns())
 
             if all_exclude_patterns:
-                self.__exclude_spec = pathspec.PathSpec.from_lines(
-                    pathspec.patterns.GitWildMatchPattern, all_exclude_patterns
-                )
+                self.__exclude_spec = pathspec.GitIgnoreSpec.from_lines(all_exclude_patterns)
 
             self.__exclude_patterns = all_exclude_patterns
 
@@ -218,9 +216,7 @@ class BuilderConfig:
                 all_artifact_patterns.append(artifact_pattern)
 
             if all_artifact_patterns:
-                self.__artifact_spec = pathspec.PathSpec.from_lines(
-                    pathspec.patterns.GitWildMatchPattern, all_artifact_patterns
-                )
+                self.__artifact_spec = pathspec.GitIgnoreSpec.from_lines(all_artifact_patterns)
 
             self.__artifact_patterns = all_artifact_patterns
 
@@ -231,6 +227,16 @@ class BuilderConfig:
         if self.__hook_config is None:
             hook_config = {}
 
+            global_hook_config = self.build_config.get('hooks', {})
+            if not isinstance(global_hook_config, dict):
+                raise TypeError('Field `tool.hatch.build.hooks` must be a table')
+
+            for hook_name, config in global_hook_config.items():
+                if not isinstance(config, dict):
+                    raise TypeError(f'Field `tool.hatch.build.hooks.{hook_name}` must be a table')
+
+                hook_config.setdefault(hook_name, config)
+
             target_hook_config = self.target_config.get('hooks', {})
             if not isinstance(target_hook_config, dict):
                 raise TypeError(f'Field `tool.hatch.build.targets.{self.plugin_name}.hooks` must be a table')
@@ -243,16 +249,6 @@ class BuilderConfig:
 
                 hook_config[hook_name] = config
 
-            global_hook_config = self.build_config.get('hooks', {})
-            if not isinstance(global_hook_config, dict):
-                raise TypeError('Field `tool.hatch.build.hooks` must be a table')
-
-            for hook_name, config in global_hook_config.items():
-                if not isinstance(config, dict):
-                    raise TypeError(f'Field `tool.hatch.build.hooks.{hook_name}` must be a table')
-
-                hook_config.setdefault(hook_name, config)
-
             final_hook_config = {}
             if not env_var_enabled(BuildEnvVars.NO_HOOKS):
                 all_hooks_enabled = env_var_enabled(BuildEnvVars.HOOKS_ENABLE)
@@ -337,6 +333,40 @@ class BuilderConfig:
 
         return self.__require_runtime_dependencies
 
+    @property
+    def require_runtime_features(self):
+        if self.__require_runtime_features is None:
+            if 'require-runtime-features' in self.target_config:
+                features_config = self.target_config
+                features_location = f'tool.hatch.build.targets.{self.plugin_name}.require-runtime-features'
+            else:
+                features_config = self.build_config
+                features_location = 'tool.hatch.build.require-runtime-features'
+
+            require_runtime_features = features_config.get('require-runtime-features', [])
+            if not isinstance(require_runtime_features, list):
+                raise TypeError(f'Field `{features_location}` must be an array')
+
+            all_features = {}
+            for i, feature in enumerate(require_runtime_features, 1):
+                if not isinstance(feature, str):
+                    raise TypeError(f'Feature #{i} of field `{features_location}` must be a string')
+                elif not feature:
+                    raise ValueError(f'Feature #{i} of field `{features_location}` cannot be an empty string')
+
+                feature = normalize_project_name(feature)
+                if feature not in self.builder.metadata.core.optional_dependencies:
+                    raise ValueError(
+                        f'Feature `{feature}` of field `{features_location}` is not defined in '
+                        f'field `project.optional-dependencies`'
+                    )
+
+                all_features[feature] = None
+
+            self.__require_runtime_features = list(all_features)
+
+        return self.__require_runtime_features
+
     @property
     def only_packages(self):
         """
@@ -500,6 +530,7 @@ class BuilderConfig:
                 dependencies[dependency] = None
 
             require_runtime_dependencies = self.require_runtime_dependencies
+            require_runtime_features = {feature: None for feature in self.require_runtime_features}
             for hook_name, config in self.hook_config.items():
                 hook_require_runtime_dependencies = config.get('require-runtime-dependencies', False)
                 if not isinstance(hook_require_runtime_dependencies, bool):
@@ -509,6 +540,31 @@ class BuilderConfig:
                 elif hook_require_runtime_dependencies:
                     require_runtime_dependencies = True
 
+                hook_require_runtime_features = config.get('require-runtime-features', [])
+                if not isinstance(hook_require_runtime_features, list):
+                    raise TypeError(f'Option `require-runtime-features` of build hook `{hook_name}` must be an array')
+
+                for i, feature in enumerate(hook_require_runtime_features, 1):
+                    if not isinstance(feature, str):
+                        raise TypeError(
+                            f'Feature #{i} of option `require-runtime-features` of build hook `{hook_name}` '
+                            f'must be a string'
+                        )
+                    elif not feature:
+                        raise ValueError(
+                            f'Feature #{i} of option `require-runtime-features` of build hook `{hook_name}` '
+                            f'cannot be an empty string'
+                        )
+
+                    feature = normalize_project_name(feature)
+                    if feature not in self.builder.metadata.core.optional_dependencies:
+                        raise ValueError(
+                            f'Feature `{feature}` of option `require-runtime-features` of build hook `{hook_name}` '
+                            f'is not defined in field `project.optional-dependencies`'
+                        )
+
+                    require_runtime_features[feature] = None
+
                 hook_dependencies = config.get('dependencies', [])
                 if not isinstance(hook_dependencies, list):
                     raise TypeError(f'Option `dependencies` of build hook `{hook_name}` must be an array')
@@ -525,6 +581,11 @@ class BuilderConfig:
                 for dependency in self.builder.metadata.core.dependencies:
                     dependencies[dependency] = None
 
+            if require_runtime_features:
+                for feature in require_runtime_features:
+                    for dependency in self.builder.metadata.core.optional_dependencies[feature]:
+                        dependencies[dependency] = None
+
             self.__dependencies = list(dependencies)
 
         return self.__dependencies
@@ -558,7 +619,9 @@ class BuilderConfig:
                         raise TypeError(f'Path for source `{source}` in field `{sources_location}` must be a string')
 
                     normalized_path = normalize_relative_path(path)
-                    if normalized_path:
+                    if normalized_path == '.':
+                        normalized_path = ''
+                    else:
                         normalized_path += os.sep
 
                     sources[normalize_relative_directory(source)] = normalized_path
@@ -636,7 +699,7 @@ class BuilderConfig:
                 only_include_config = self.build_config
                 only_include_location = 'tool.hatch.build.only-include'
 
-            only_include = only_include_config.get('only-include', []) or self.packages
+            only_include = only_include_config.get('only-include', self.default_only_include()) or self.packages
             if not isinstance(only_include, list):
                 raise TypeError(f'Field `{only_include_location}` must be an array')
 
@@ -723,8 +786,11 @@ class BuilderConfig:
     def default_packages(self):
         return []
 
+    def default_only_include(self):
+        return []
+
     def default_global_exclude(self):
-        patterns = ['*.py[cdo]', *EXCLUDED_DIRECTORIES]
+        patterns = ['*.py[cdo]', f'/{DEFAULT_BUILD_DIRECTORY}']
         patterns.sort()
         return patterns
 
@@ -739,9 +805,7 @@ class BuilderConfig:
             # Include anything the hooks indicate
             build_artifacts = build_data['artifacts']
             if build_artifacts:
-                self.build_artifact_spec = pathspec.PathSpec.from_lines(
-                    pathspec.patterns.GitWildMatchPattern, build_artifacts
-                )
+                self.build_artifact_spec = pathspec.GitIgnoreSpec.from_lines(build_artifacts)
 
             self.build_force_include.update(normalize_inclusion_map(build_data['force_include'], self.root))
 
